import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.BasicStroke;
import java.awt.geom.Ellipse2D;
import java.util.Random;

public class GameOfLife
{
	public GameOfLife( int w, int h)
	{
		width = w;
		height = h;
		grid = new Grid( height, width);
		generations = 0;
		original = new int[height+2][width+2];
		next = new int[height+2][width+2];
		generator = new Random();
	}
	public void draw(Graphics2D g2)
	{
		g2.setStroke(new BasicStroke(0));
		grid.draw(g2);
		for (int r = 1; r <= height; r++)
			for (int c = 1; c <= width; c++)
				if (original[r][c]==1)
					g2.fill(new Ellipse2D.Double((double)c-.8,(double)r-.8
					,.6,.6));
	}
	public void next()
	{
		next = new int[height+2][width+2];
		for (int r = 1; r <= height; r++)
			for (int c = 1; c <= width; c++)
				if ((getNeighbors(r,c) == 3)|| 
					(getNeighbors(r,c) == 2 && original[r][c] == 1))
					next[r][c]=1;
				else
					next[r][c] = 0;
		original = new int[height+2][width+2];
		for (int r = 1; r <= height; r++)
			for (int c = 1; c <= width; c++)
				original[r][c] = next[r][c];
		generations++;
	}
	public int getGenerations()
	{
		return generations;
	}
	public void set(int r, int c)
	{
		original[r][c] = original[r][c]*-1+1;
	}
	public void clear()
	{
		for (int r = 1; r <= height; r++)
			for (int c = 1; c <= width; c++)
				original[r][c] = 0;
	}
	public void random()
	{
		original[generator.nextInt(width)+1][generator.nextInt(width)+1] = 1;
		original[generator.nextInt(width)+1][generator.nextInt(width)+1] = 1;
		original[generator.nextInt(width)+1][generator.nextInt(width)+1] = 1;
		original[generator.nextInt(width)+1][generator.nextInt(width)+1] = 1;
		original[generator.nextInt(width)+1][generator.nextInt(width)+1] = 1;
		original[generator.nextInt(width)+1][generator.nextInt(width)+1] = 1;
		original[generator.nextInt(width)+1][generator.nextInt(width)+1] = 1;
	}
	private int getNeighbors(int r, int c)
	{
		return original[r-1][c-1]+original[r-1][c]+original[r-1][c+1]
		+original[r][c-1]+original[r][c+1]+original[r+1][c-1]+
		original[r+1][c]+original[r+1][c+1];
	}
	
	Grid grid;
	int width;
	int height;
	int generations;
	int [][] original;
	int [][] next;
	private Random generator;
}