import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.BasicStroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.GridLayout;
import java.applet.Applet;
import javax.swing.Timer;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.SwingConstants;
import javax.swing.JComboBox;

public class Life extends Applet
{
	public Life()
	{
		life = new GameOfLife( DEFAULTH , DEFAULTW );
		delay=500;
		makeButtons();
		makeComboBoxes();
		makePanel();
		makeFrame();
		makeMouse();
	}
	public void paint(Graphics g)
	{
		Graphics2D g2=(Graphics2D)g;
		double xscale = (getWidth() - 1.0) / DEFAULTW;
		double yscale = (getHeight() - 1.0) / DEFAULTH;
		g2.scale(xscale, yscale);
		g2.translate(0, 0);
		life.draw(g2);
	}
	private void makeButtons()
	{
		class Advance implements ActionListener
		{
			public void actionPerformed(ActionEvent event)
			{
				life.next();
				field.setText(String.valueOf(life.getGenerations()));
				repaint();
			}
		}
		buttonNext = new JButton("Next");
		buttonNext.addActionListener(new Advance());
		
		t=new Timer(delay, new Advance());
		class Go implements ActionListener
		{
			public void actionPerformed(ActionEvent event)
			{
				String name = buttonGo.getText();
				if (name.equals("Go"))
				{
					buttonGo.setText("Stop");
					t.start();
				}
				else
				{
					buttonGo.setText("Go");
					t.stop();
				}
			}
		}
		buttonGo = new JButton("Go");
		buttonGo.addActionListener(new Go());
		
		class Clear implements ActionListener
		{
			public void actionPerformed(ActionEvent event)
			{
				life.clear();
				repaint();
			}
		}
		buttonClear = new JButton("Clear");
		buttonClear.addActionListener(new Clear());
		
		class Rand implements ActionListener
		{
			public void actionPerformed(ActionEvent event)
			{
				life.random();
				repaint();
			}
		}
		buttonRand = new JButton("Random");
		buttonRand.addActionListener(new Rand());
	}
	private void makePanel()
	{
		label = new JLabel("Generations: ",SwingConstants.RIGHT);
		
		field = new JTextField(8);
		field.setText(String.valueOf(life.getGenerations()));
		field.setEditable(false);
		
		panel = new JPanel(new GridLayout(3,3));
		panel.add(buttonNext);
		panel.add(buttonGo);
		panel.add(buttonClear);
		panel.add(speed);
		panel.add(buttonRand);
		panel.add(pattern);
		panel.add(label);
		panel.add(field);
		panel.add(new JLabel("By Chad Link",SwingConstants.CENTER));
	}
	private void makeFrame()
	{
		frame = new JFrame();
		frame.setContentPane(panel);
		frame.pack();
		frame.show();
	}
	private void makeMouse()
	{
		class MousePressListener implements MouseListener
		{  
			public void mousePressed(MouseEvent event)
			{  
            	int x = (int)(event.getX()*DEFAULTW / (getWidth() - 1.0))+1;
            	int y = (int)(event.getY()*DEFAULTH / (getHeight() - 1.0))+1;
				life.set(y,x);
				repaint();
			}
			public void mouseReleased(MouseEvent event) {}
			public void mouseClicked(MouseEvent event) {}
			public void mouseEntered(MouseEvent event) {}
			public void mouseExited(MouseEvent event) {}
		}
		addMouseListener(new MousePressListener());
	}
	private void makeComboBoxes()
	{
		class Speed implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				String s = (String) speed.getSelectedItem();
				if (s.equals("Slow"))
					t.setDelay(500);
				else if (s.equals("Medium"))
					t.setDelay(150);
				else if (s.equals("Fast"))
					t.setDelay(50);
				else 
					t.setDelay(10);
			}
		}
		speed = new JComboBox();
		speed.addItem("Slow");
		speed.addItem("Medium");
		speed.addItem("Fast");
		speed.addItem("Warp Speed");
		speed.addActionListener(new Speed());
		
		class Pat implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				String s = (String) pattern.getSelectedItem();
				life.clear();
				if (s.equals("Glider"))
				{
					life.set(1,2);
					life.set(2,3);
					life.set(3,1);
					life.set(3,2);
					life.set(3,3);
				}
				else if (s.equals("Explosion"))
				{
					life.set(28,28);
					life.set(29,28);
					life.set(30,28);
					life.set(31,28);
					life.set(32,28);
					life.set(28,30);
					life.set(32,30);
					life.set(28,32);
					life.set(29,32);
					life.set(30,32);
					life.set(31,32);
					life.set(32,32);	
				}
				else
				{
					life.set(23,5);
					life.set(23,6);
					life.set(24,5);
					life.set(24,6);
					life.set(28,7);
					life.set(28,8);
					life.set(33,3);
					life.set(33,4);
					life.set(33,8);
					life.set(33,9);
					life.set(34,4);
					life.set(34,5);
					life.set(34,6);
					life.set(34,7);
					life.set(34,8);
					life.set(35,4);
					life.set(35,5);
					life.set(35,7);
					life.set(35,8);
					life.set(36,4);
					life.set(36,5);
					life.set(36,7);
					life.set(36,8);
					life.set(37,5);
					life.set(37,6);
					life.set(37,7);
					life.set(42,7);
					life.set(42,8);
					life.set(42,9);
					life.set(43,7);
					life.set(43,8);
					life.set(43,9);
					life.set(44,6);
					life.set(44,10);
					life.set(45,5);
					life.set(45,11);
					life.set(46,6);
					life.set(46,10);
					life.set(47,7);
					life.set(47,8);
					life.set(47,9);
					life.set(57,7);
					life.set(57,8);
					life.set(58,7);
					life.set(58,8);
				}
				repaint();
			}
		}
		pattern = new JComboBox();
		pattern.addItem("Glider");
		pattern.addItem("Explosion");
		pattern.addItem("Glider Gun");
		pattern.addActionListener(new Pat());
	}
	
	private GameOfLife life;
	private final int DEFAULTH=60;
	private final int DEFAULTW=60;
	private JComboBox speed;
	private JComboBox pattern;
	private JButton buttonGo;
	private JButton buttonNext;
	private JButton buttonClear;
	private JButton buttonRand;
	private JLabel label;
	private JTextField field;
	private JPanel panel;
	private JFrame frame;
	private Timer t;
	private int delay;
}